# -*- coding: utf-8 -*-
"""
Created on Sun Sep 26 09:34:44 2021

@author: mumuz
"""

ERROR_INVALID_ID = 'Invalid ID: {}'
ERROR_INVALID_NUM = 'Invalid number: {}'
ERROR_IDEXIST = 'ID: {} exists'
ERROR_IDNOTEXIST = 'ID: {} does not exist'
ERROR_POLYGONLESSVERTEX = 'Polygon has less than 3 points'
ERROR_POLYLINELESSVERTEX = 'Polyline has less than 2 points'